from datetime import datetime, timedelta
from typing import Union

from fastapi import (
    APIRouter,
    Cookie,
    Depends,
    Form,
    HTTPException,
    Response,
    Security,
    status,
)
from fastapi.responses import JSONResponse
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy import delete, or_, select

from .. import __version__
from ..core.config import settings
from ..core.security import (
    Token,
    authenticate_user,
    client_allowed,
    create_access_token,
    get_current_active_user,
)
from ..db.session import SessionFuture
from ..models.client import Client
from ..models.login_history import LoginHistory
from ..schemas.user import UserToken


router = APIRouter()


@router.post("/token", response_model=Token)
async def login_for_access_token(
    response: Response,
    form_data: OAuth2PasswordRequestForm = Depends(),
    client_id: int = Cookie(None),
    otp: int = Form(None),
) -> Union[Token, JSONResponse]:
    with SessionFuture() as db:
        user = authenticate_user(form_data.username, form_data.password, db)
        if not user:
            raise HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED,
                detail="Incorrect username or password",
                headers={"WWW-Authenticate": "Bearer"},
            )
        allowed, client = client_allowed(user, client_id, otp, db)
        db.flush()
        if allowed:
            history = LoginHistory(user.id, client.id)
            db.add(history)
            db.flush()
        db.execute(
            delete(LoginHistory)
            .where(
                or_(
                    LoginHistory.date < datetime.utcnow() - timedelta(days=30),
                    LoginHistory.client_id.in_(
                        select(Client.id)
                        .where(
                            Client.creation_date < datetime.utcnow() - timedelta(days=3),
                            Client.enabled == False,  # noqa: E712
                        )
                        .subquery()
                    ),
                )
            )
            .execution_options(synchronize_session=False)
        )
        db.execute(
            delete(Client).where(
                Client.creation_date < datetime.utcnow() - timedelta(days=3), Client.enabled == False  # noqa: E712
            )
        )
        db.commit()
        response.set_cookie(key="client_id", value=str(client.code), max_age=10 * 365 * 24 * 60 * 60)
        if not allowed:
            not_allowed_response = JSONResponse(
                status_code=status.HTTP_401_UNAUTHORIZED,
                headers={"WWW-Authenticate": "Bearer"},
                content={"detail": "Client is not registered"},
            )
            not_allowed_response.set_cookie(key="client_id", value=str(client.code), max_age=10 * 365 * 24 * 60 * 60)
            return not_allowed_response
        access_token_expires = timedelta(minutes=settings.JWT_TOKEN_EXPIRE_MINUTES)
        access_token = create_access_token(
            data={
                "sub": user.name,
                "scopes": ["authenticated"]
                + list(
                    set([p.name.replace(" ", "-").lower() for r in user.roles for p in r.permissions])
                ),  # noqa: W503
                "userId": str(user.id),
                "lockedOut": user.locked_out,
                "ver": __version__.__version__,
            },
            expires_delta=access_token_expires,
        )
        return Token(access_token=access_token, token_type="bearer")


@router.post("/refresh", response_model=Token)
async def refresh_token(user: UserToken = Security(get_current_active_user)) -> Token:
    access_token_expires = timedelta(minutes=settings.JWT_TOKEN_EXPIRE_MINUTES)
    access_token = create_access_token(
        data={
            "sub": user.name,
            "scopes": user.permissions,
            "userId": str(user.id_),
            "lockedOut": user.locked_out,
            "ver": __version__.__version__,
        },
        expires_delta=access_token_expires,
    )
    return Token(access_token=access_token, token_type="bearer")
